/*============================================================================
  File:     PairMatching.sql

  Summary:  Shows catching sp_statement_starting events that don't have
			a matching sp_statement_completed event

  Date:     March 2011

  SQL Server Version: 10.0.2531.0 (SQL Server 2008 SP1)
------------------------------------------------------------------------------
  Written by Jonathan M. Kehayias, Paul S. Randal, SQLskills.com

  (c) 2011, SQLskills.com. All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you include this copyright and give due
  credit, but you must obtain prior permission before blogging this code.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

-- Create an example proc
USE msdb;
GO
IF OBJECT_ID ('StoredProcedureExceedsDuration') IS NOT NULL
	DROP PROCEDURE dbo.StoredProcedureExceedsDuration;
GO
IF OBJECT_ID ('StoredProcedureExceedsDuration2') IS NOT NULL
	DROP PROCEDURE dbo.StoredProcedureExceedsDuration2;
GO

CREATE PROCEDURE dbo.StoredProcedureExceedsDuration2 (
	@WaitForValue VARCHAR (12) = '00:00:00:050')
AS
BEGIN
	WAITFOR DELAY @WaitForValue; 
END
GO
CREATE PROCEDURE dbo.StoredProcedureExceedsDuration (
	@WaitForValue VARCHAR (12) = '00:00:00:050')
AS
BEGIN
	EXEC StoredProcedureExceedsDuration2 @WaitForValue; 
END
GO

-- If the event Session exists drop it
IF EXISTS (SELECT * FROM sys.server_event_sessions 
	WHERE name='StatementTimeouts') 
    DROP EVENT SESSION [StatementTimeouts] ON SERVER; 

-- Create the Event Session
CREATE EVENT SESSION [StatementTimeouts]
ON SERVER
ADD EVENT sqlserver.sp_statement_starting
(
	ACTION (
		sqlserver.session_id,
		sqlserver.tsql_stack,
		sqlserver.sql_text,
		sqlserver.plan_handle)
	WHERE (state = 0) -- Don't fire Recompiled Events
),
ADD EVENT sqlserver.sp_statement_completed
(
	ACTION (
		sqlserver.session_id,
		sqlserver.tsql_stack)
)
ADD TARGET package0.pair_matching
(
	SET begin_event = 'sqlserver.sp_statement_starting',
		begin_matching_actions = 'sqlserver.session_id, sqlserver.tsql_stack',
		end_event = 'sqlserver.sp_statement_completed',
		end_matching_actions = 'sqlserver.session_id, sqlserver.tsql_stack'
)
WITH (MAX_DISPATCH_LATENCY=1 SECONDS, TRACK_CAUSALITY = ON);
GO

-- Start the Event Session
ALTER EVENT SESSION StatementTimeouts
ON SERVER STATE = START;
GO

-- Change connection properties to have a 10
-- second timeout

-- Run a test workload
USE msdb;
GO
-- These won't time out
EXECUTE StoredProcedureExceedsDuration;
GO 10
-- This will...
EXECUTE StoredProcedureExceedsDuration '00:00:30.000';
GO
-- These won't time out
EXECUTE StoredProcedureExceedsDuration;
GO 10

-- Drop the Event to stop collection
ALTER EVENT SESSION StatementTimeouts
ON SERVER
DROP EVENT sqlserver.sp_statement_starting,
DROP EVENT sqlserver.sp_statement_completed
GO

-- Change connection properties back again

-- Query the XML to get the Target Data
SELECT 
	statements.value (
		'(@name)[1]', 'varchar(50)') AS [Event], 
	statements.value (
		'(@timestamp)[1]', 'datetime2') AS [Time], 
	statements.value (
		'(action[@name="session_id"]/value)[1]', 'int') AS [Session],
	statements.value (
		'(data[@name="source_database_id"]/value)[1]', 'int') AS [DbId],
	statements.value (
		'(data[@name="offset"]/value)[1]', 'int') AS [Offset],
	statements.value (
		'(data[@name="offset_end"]/value)[1]', 'int') AS [OffsetEnd],
	statements.value (
		'(data[@name="nest_level"]/value)[1]', 'int') AS [NestLevel], 
	SUBSTRING (statements.value (
		'(action[@name=''plan_handle'']/value)[1]', 'VARCHAR(100)'), 15, 50) AS [Plan], 
	CAST (statements.value (
		'(action[@name="tsql_stack"]/value)[1]', 'nvarchar(4000)') AS XML) AS [Stack], 
	statements.value (
		'(action[@name="sql_text"]/value)[1]', 'nvarchar(max)') AS [Text],
	statements.value (
		'(action[@name="attach_activity_id"]/value)[1]', 'uniqueidentifier') AS [ActivityId], 
	CAST (SUBSTRING (statements.value (
		'(action[@name="attach_activity_id"]/value)[1]', 'varchar(50)'), 38, 10) AS INT) AS [Sequence]
FROM
(
	SELECT
		CAST (target_data AS XML) as data
	FROM sys.dm_xe_sessions AS s 
	JOIN sys.dm_xe_session_targets AS t 
		ON t.event_session_address = s.address
	WHERE
		s.name = 'StatementTimeouts'
		AND t.target_name = 'pair_matching'
) AS data
CROSS APPLY data.nodes ('PairingTarget/event') AS q (statements);
GO

-- Drop the Event Session
DROP EVENT SESSION StatementTimeouts
ON SERVER;
GO